import adsk.core, adsk.fusion, adsk.cam, traceback, os, platform
from .MainDialogInputChangeHandler import MainDialogInputChangeHandler
from .DecorItButtonExecuteEventHandler import DecorItButtonExecuteEventHandler
from ..Static import Static


class DecorItButtonCreateEventHandler(adsk.core.CommandCreatedEventHandler):
    def __init__(self):
        super().__init__()
        self.onExecute = None
    
    def __init__(self, add_handler):
        super().__init__()
        self.add_handler = add_handler

    def populate_table(self, table):
        table.minimumVisibleRows = 3
        table.maximumVisibleRows = 6
        table.columnSpacing = 1
        table.rowSpacing = 1
        table.tablePresentationStyle = adsk.core.TablePresentationStyles.itemBorderTablePresentationStyle
        table.hasGrid = False

        cmdInputs = adsk.core.CommandInputs.cast(table.commandInputs)
        img = cmdInputs.addImageCommandInput('chair_img', '', "resources/iChair.png")
        txt = cmdInputs.addStringValueInput('chair', 'Chair', 'Chair')
        txt.isReadOnly = True
        table.addCommandInput(img, 1, 0)
        table.addCommandInput(txt, 1, 1)
        # img.toolClipFilename = "resources/Chair.png"

        img = cmdInputs.addImageCommandInput('shelf_img', '', "resources/iShelf.png")
        txt = cmdInputs.addStringValueInput('shelf', 'Shelf', 'Shelf')
        txt.isReadOnly = True
        table.addCommandInput(img, 2, 0)
        table.addCommandInput(txt, 2, 1)

        img = cmdInputs.addImageCommandInput('table_img', '', "resources/iTable.png")
        txt = cmdInputs.addStringValueInput('table', 'Table', 'Table')
        txt.isReadOnly = True
        table.addCommandInput(img, 3, 0)
        table.addCommandInput(txt, 3, 1)

    def populate_tools_wall_dropdown(self, inputs):
        index = 1
        for i in range(20):
            textBox = inputs.addTextBoxCommandInput('room' + str(index), '', '<b>Room ' + str(index) + '</b>', 1, True)
            # textBox.isVisible = False
            textBox.isReadOnly = True

            dropDown = inputs.addDropDownCommandInput('tools_wall_select' + str(index), 'Walls', adsk.core.DropDownStyles.TextListDropDownStyle)
            dropDown.listItems.add('All', Static.mode == "All", '')
            dropDown.listItems.add('Default', Static.mode == "Default", '')
            dropDown.listItems.add('None', Static.mode == "None", '')
            # dropDown.isVisible = False

            index += 1

    def notify(self, args):
        try:
            eventArgs = adsk.core.CommandCreatedEventArgs.cast(args)
            cmd = eventArgs.command
            cmd.okButtonText = "Create"
            cmd.cancelButtonText = "Close"

            # Creating dialog with the following inputs
            inputs = cmd.commandInputs
            mainTab = inputs.addTabCommandInput('tab_main', 'Room')
            roomTab = inputs.addTabCommandInput('tab_room', 'Modify')
            furnitureTab = inputs.addTabCommandInput('tab_furniture', 'Furniture')
            infoTab = inputs.addTabCommandInput('tab_info', 'Info')
            toolsTab = inputs.addTabCommandInput('tab_tools', 'Tools')

            mainTabChildInputs = mainTab.children
            mainTabChildInputs.addValueInput('floor_length_input', 'Floor Length (m)', 'm', adsk.core.ValueInput.createByReal(500))
            mainTabChildInputs.addValueInput('floor_width_input', 'Floor Width (m)', 'm', adsk.core.ValueInput.createByReal(500))
            mainTabChildInputs.addValueInput('floor_height_input', 'Wall Height (m)', 'm', adsk.core.ValueInput.createByReal(300))

            selectFloor = mainTabChildInputs.addSelectionInput('floor_selection', 'Select A Wall', 'Please Select a Wall to place new Romms')
            selectFloor.addSelectionFilter(adsk.core.SelectionCommandInput.Bodies)
            selectFloor.setSelectionLimits(0,1)
            selectFloor.isVisible = Static.count() > 0

            # dropDown = mainTabChildInputs.addDropDownCommandInput('floor_place_dropdown', 'Position', adsk.core.DropDownStyles.TextListDropDownStyle)
            # dropDown.listItems.add('Top', False, '')
            # dropDown.listItems.add('Bottom', False, '')
            # dropDown.listItems.add('Left', True, '')
            # dropDown.listItems.add('Right', False, '')
            # dropDown.isVisible = Static.count() > 0

            furnitureTabChildrenInputs = furnitureTab.children

            table = furnitureTabChildrenInputs.addTableCommandInput('furniture_table', 'Furniture', 2, '1:3')
            self.populate_table(table)

            infoTabChildrenInputs = infoTab.children
            toolsTabChildrenInputs = toolsTab.children

            dropDown = toolsTabChildrenInputs.addDropDownCommandInput('tools_wall_select', 'Walls', adsk.core.DropDownStyles.TextListDropDownStyle)
            dropDown.listItems.add('All', False, '')
            dropDown.listItems.add('Default', True, '')
            dropDown.listItems.add('None', False, '')

            # self.populate_tools_wall_dropdown(toolsTabChildrenInputs)
            # table2 = infoTabChildrenInputs.addTableCommandInput('info_table', 'Info', 2, '2:1')

            # dropDown = furnitureTabChildrenInputs.addDropDownCommandInput('furniture_select', 'Select Furniture', adsk.core.DropDownStyles.TextListDropDownStyle)
            # dropDown.listItems.add('Chair', False, '')
            # dropDown.listItems.add('Shelf', False, '')
            # dropDown.listItems.add('Table', False, '')


            roomTabChildInputs = roomTab.children

            dropDown = roomTabChildInputs.addDropDownCommandInput('type_select', 'Select Type', adsk.core.DropDownStyles.TextListDropDownStyle)
            dropDown.listItems.add('Window', True, '')
            dropDown.listItems.add('Door', False, '')

            selectWall = roomTabChildInputs.addSelectionInput('wall_selection', 'Select Wall', 'Please Select a wall to place door/window')
            selectWall.addSelectionFilter(adsk.core.SelectionCommandInput.Faces)
            selectWall.setSelectionLimits(0,1)

            dropDown2 = roomTabChildInputs.addDropDownCommandInput('position_select', 'Select Position', adsk.core.DropDownStyles.TextListDropDownStyle)
            dropDown2.listItems.add('Centre', True, '')
            dropDown2.listItems.add('Right', False, '')
            dropDown2.listItems.add('Left', False, '')

            roomTabChildInputs.addValueInput('windowOrDoor_height_input', 'Height (m)', 'm',adsk.core.ValueInput.createByReal(100))
            roomTabChildInputs.addValueInput('windowOrDoor_width_input', 'Width (m)', 'm',adsk.core.ValueInput.createByReal(100))

            selectInput = furnitureTabChildrenInputs.addSelectionInput('furniture_face_selection', 'Wall (Optional)', 'Please select a wall to pleace the furniture')
            selectInput.addSelectionFilter(adsk.core.SelectionCommandInput.Faces)
            selectInput.setSelectionLimits(0, 1)

            onInputChanged = MainDialogInputChangeHandler()
            cmd.inputChanged.add(onInputChanged)
            self.add_handler(onInputChanged)

            onExecute = DecorItButtonExecuteEventHandler()
            cmd.execute.add(onExecute)
            self.add_handler(onExecute)

        except:
            if Static.ui:
                Static.ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

