from ..Static import Static
import adsk.core, adsk.fusion, adsk.cam, traceback

class CreateFloor:
    def __init__(self) -> None:
        pass
    

    def change_camera(self):
        try:
            camera = Static.app.activeViewport.camera
            # camera.viewOrientation = adsk.core.ViewOrientations.FrontViewOrientation
            camera.isFitView = True
            Static.app.activeViewport.camera = camera
        except:
            if Static.ui:
                Static.ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

    def merge_wall(self, _occ):
        try:
            product = Static.app.activeProduct
            rootComp = product.rootComponent
            for _body in _occ.bRepBodies:
                if "Wall" in _body.name:
                    boundingBox = _body.boundingBox
                    # centre_x = (boundingBox.maxPoint.x + boundingBox.minPoint.x) / 2 + 5
                    # centre_y = (boundingBox.maxPoint.y + boundingBox.minPoint.y) / 2 + 5
                    # centre_z = (boundingBox.maxPoint.z + boundingBox.minPoint.z) / 2
                    # point = adsk.core.Point3D.create(centre_x, centre_y, centre_z)
                    for occ in rootComp.occurrences:
                        if occ.name == _occ.name:
                            continue
                        for body in occ.bRepBodies:
                            if body.boundingBox.intersects(boundingBox):
                                print(_body.name)

        except:
            if Static.ui:
                Static.ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

    def shift_component(self, _occ, floor, length, width):
        try:
            product = Static.app.activeProduct
            rootComp = product.rootComponent
            design = adsk.fusion.Design.cast(product)
            
            boundingBox = floor.boundingBox
            floor_comp = floor.parentComponent
            if "Floor" in floor_comp.name:
                entity = design.findEntityByToken(floor.entityToken)[0]
                bounding_box = entity.boundingBox
                translate_x = (bounding_box.maxPoint.x + bounding_box.minPoint.x) * 0.5
                translate_y = (bounding_box.maxPoint.y + bounding_box.minPoint.y) * 0.5


            if floor.name == "Wall 1":
                distance = length * 0.5 + 7.5
                # distance = abs(floor_bounding_box.maxPoint.x - floor_bounding_box.minPoint.x) * 0.5
                translate_x = boundingBox.maxPoint.x - distance
                _occ.bRepBodies.itemByName("Wall 4").deleteMe()
            elif floor.name == "Wall 4":
                distance = length * 0.5 - 7.5
                # distance = abs(floor_bounding_box.maxPoint.x - floor_bounding_box.minPoint.x) * 0.5
                translate_x = boundingBox.maxPoint.x + distance
                _occ.bRepBodies.itemByName("Wall 1").deleteMe()
            elif floor.name == "Wall 2":
                distance = width * 0.5 - 7.5
                # distance = abs(floor_bounding_box.maxPoint.y - floor_bounding_box.minPoint.y) + (abs(floor_bounding_box.maxPoint.y - floor_bounding_box.minPoint.y) + width / 2) * 0.5
                translate_y = boundingBox.maxPoint.y + distance
                _occ.bRepBodies.itemByName("Wall 3").deleteMe()
            elif floor.name == "Wall 3":
                distance = width * 0.5 + 7.5
                # distance = abs(floor_bounding_box.maxPoint.y - floor_bounding_box.minPoint.y) * 0.5
                translate_y = boundingBox.maxPoint.y - distance
                _occ.bRepBodies.itemByName("Wall 2").deleteMe()
            else:
                if Static.ui:
                    Static.ui.messageBox('Invalid Body Selected')
                return

            existing_transform = _occ.transform
            existing_translation = existing_transform.translation

            existing_translation.x += translate_x
            existing_translation.y += translate_y

            modified_transform = existing_transform
            modified_transform.translation = adsk.core.Vector3D.create(existing_translation.x, existing_translation.y, existing_translation.z)
            _occ.transform = modified_transform
            Static.app.activeViewport.refresh()


        except:
            if Static.ui:
                Static.ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))


    def draw_floor(self, sketch, lines, extrudes, length, width):
        FLOOR_HEIGHT = 15
        APPEARANCE_NAME = "Bamboo Light - Semigloss"

        lines.addTwoPointRectangle(adsk.core.Point3D.create(- length / 2, width / 2, 0), adsk.core.Point3D.create(length /2, - width / 2, 0))
        
        prof = sketch.profiles.item(len(sketch.profiles) - 1)
        distance = adsk.core.ValueInput.createByReal(FLOOR_HEIGHT)
        extrude1 = extrudes.addSimple(prof, distance, adsk.fusion.FeatureOperations.NewBodyFeatureOperation)
        
        body1 = extrude1.bodies.item(0)
        body1.name = "Floor 0"

        fusionAppearence = Static.app.materialLibraries.itemByName('Fusion 360 Appearance Library')
        appearance = fusionAppearence.appearances.itemByName(APPEARANCE_NAME)
        body1.appearance = appearance

    def draw_wall(self, sketch, lines, extrudes, x1, y1, x2, y2, index, height, visible=True):
        # WALL_HEIGHT = 300
        APPEARANCE_NAME = "Paint - Enamel Glossy (White)"
        WALL_NAME = "Wall " + index

        point1 = adsk.core.Point3D.create(x1, y1, 15)
        point2 = adsk.core.Point3D.create(x2, y2, 15)
        lines.addTwoPointRectangle(point1, point2)

        prof = sketch.profiles.item(len(sketch.profiles) - 1)
        for p in sketch.profiles:
            if p.boundingBox.minPoint.x == x1 and p.boundingBox.minPoint.y == y1 and p.boundingBox.maxPoint.x == x2 and p.boundingBox.maxPoint.y == y2:
                prof = p
                break
            if p.boundingBox.minPoint.x == x2 and p.boundingBox.minPoint.y == y2 and p.boundingBox.maxPoint.x == x1 and p.boundingBox.maxPoint.y == y1:
                prof = p
                break
        
        distance = adsk.core.ValueInput.createByReal(height)
        extrude1 = extrudes.addSimple(prof, distance, adsk.fusion.FeatureOperations.NewBodyFeatureOperation)

        body1 = extrude1.bodies.item(0)
        body1.name = WALL_NAME

        fusionAppearence = Static.app.materialLibraries.itemByName('Fusion 360 Appearance Library')
        appearance = fusionAppearence.appearances.itemByName(APPEARANCE_NAME)
        body1.appearance = appearance
        body1.isLightBulbOn = visible

    def createFloor(self, inputs):
        length = inputs.itemById("floor_length_input").value
        width = inputs.itemById("floor_width_input").value
        height = inputs.itemById("floor_height_input").value
        selected_floor = inputs.itemById("floor_selection").selection(0).entity if inputs.itemById("floor_selection").selectionCount > 0 else None

        try:
            product = Static.app.activeProduct
            rootComp = product.rootComponent
            if Static.count() > 1:
                if not selected_floor:
                    if Static.ui:
                        Static.ui.messageBox('Please select a wall to place the new room on.')
                    return

            floor_occ = rootComp.occurrences.addNewComponent(adsk.core.Matrix3D.create())
            flooc_comp = floor_occ.component
            flooc_comp.name = "Floor"

            extrudes = flooc_comp.features.extrudeFeatures

            sketches = flooc_comp.sketches
            xyPlane = flooc_comp.xYConstructionPlane
            sketch = sketches.add(xyPlane)
            lines = sketch.sketchCurves.sketchLines

            WALL_THICKNESS = 15
            x = length / 2 - 7.5
            y = width / 2 - 7.5

            self.draw_floor(sketch, lines, extrudes, length, width)
            self.draw_wall(sketch, lines, extrudes, - x, y, - x - WALL_THICKNESS, - y, "1", height, Static.mode != "None")
            self.draw_wall(sketch, lines, extrudes, - x, y, x, y + WALL_THICKNESS, "2", height, Static.mode != "None")
            self.draw_wall(sketch, lines, extrudes, x, - y, - x, - y - WALL_THICKNESS, "3", height, Static.mode == "All")
            self.draw_wall(sketch, lines, extrudes, x, - y, x + WALL_THICKNESS, y, "4", height, Static.mode == "All")

            if Static.count() > 1:
                self.shift_component(floor_occ, selected_floor, length, width)
                # self.merge_wall(floor_occ)
            
            self.change_camera()
            # Static.tokens.append(floor_occ.entityToken)

        except:
            if Static.ui:
                Static.ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))
